@@ -10,7 +10,7 @@ parameters = ['P0', 'P1', 'P2', 'P3', 'P4']
 
 def values(x, p):
     p0, p1, p2, p3, p4 = p
-    return p0 + p1*x + p2*x**2 + p3*x**3 + p4*x**5
+    return p0 + x*(p1 + x*(p2 + x*(p3 + x*p4)))
 
 def guess(x, y):
     slope = (y[-1]-y[0]) / (x[-1]-x[0])
